/*
 * Copyright 2008,2009 WSO2, Inc. http://www.wso2.org.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wso2.mercury.modules;

import org.apache.axis2.modules.Module;
import org.apache.axis2.context.ConfigurationContext;
import org.apache.axis2.description.AxisModule;
import org.apache.axis2.description.AxisDescription;
import org.apache.axis2.AxisFault;
import org.apache.axis2.Constants;
import org.apache.neethi.Policy;
import org.apache.neethi.Assertion;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.mercury.util.MercuryConstants;
import org.wso2.mercury.state.*;
import org.wso2.mercury.workers.RMSSequenceWorker;
import org.wso2.mercury.workers.RMDSequenceWorker;
import org.wso2.mercury.workers.InvokerWorker;
import org.wso2.mercury.workers.SequenceRemovalWorker;
import org.wso2.mercury.persistence.PersistenceManager;
import org.wso2.mercury.handlers.MercuryOutHandler;

/**
 * This module provides the WS-RM functionality to Axis2.
 */

public class MercuryModule implements Module {

     private static Log log = LogFactory.getLog(MercuryModule.class);
    // initialize the module
    public void init(ConfigurationContext configContext,
                     AxisModule module) throws AxisFault {

        RMSSequence.TIMEOUT_TIME =
                Long.parseLong((String)module.getParameter(MercuryConstants.RMS_SEQUENCE_TIMEOUT).getValue());
        RMSSequence.RETRANSMIT_TIME =
                Long.parseLong((String)module.getParameter(MercuryConstants.RMS_SEQUENCE_RETRANSMIT_TIME).getValue());
        RMSSequenceWorker.SEQUENCE_WORKER_SLEEP_TIME =
                Long.parseLong((String)module.getParameter(MercuryConstants.RMS_SEQUENCE_WORKER_SLEEP_TIME).getValue());

        RMDSequence.TIMEOUT_TIME =
                Long.parseLong((String)module.getParameter(MercuryConstants.RMD_SEQUENCE_TIMEOUT).getValue());
        RMDSequence.RETRANSMIT_TIME =
                Long.parseLong((String)module.getParameter(MercuryConstants.RMD_SEQUENCE_RETRANSMIT_TIME).getValue());
        RMDSequenceWorker.SEQUENCE_WORKER_SLEEP_TIME =
                Long.parseLong((String)module.getParameter(MercuryConstants.RMD_SEQUENCE_WORKER_SLEEP_TIME).getValue());

        InvokerBuffer.TIMEOUT_TIME =
                Long.parseLong((String)module.getParameter(MercuryConstants.INVOKER_TIMEOUT).getValue());
        InvokerWorker.INVOKER_WORKER_SLEEP_TIME =
                Long.parseLong((String)module.getParameter(MercuryConstants.INVOKER_WORKER_SLEEP_TIME).getValue());

        long sequenceRemvalWorkerSleepTime =
                Long.parseLong((String)module.getParameter(MercuryConstants.SEQUENCE_REMOVAL_WORKER_SLEEP_TIME).getValue());

        // register two new context objects
        configContext.setProperty(MercuryConstants.RMS_CONTEXT, new RMSContext(configContext));
        configContext.setProperty(MercuryConstants.RMD_CONTEXT, new RMDContext(configContext));

        String enforceRM = (String) module.getParameter(MercuryConstants.ENFORCE_RM).getValue();
        if (Constants.VALUE_TRUE.equals(enforceRM)) {
            configContext.setProperty(MercuryConstants.ENFORCE_RM, Constants.VALUE_TRUE);
        }

        //initialize the persistence storage manager and stored save it in
        // configuratin context
        if (module.getParameter(MercuryConstants.RM_PERSISTANCE_MANAGER) != null) {
            String className = (String) module.getParameter(MercuryConstants.RM_PERSISTANCE_MANAGER).getValue();
            try {
                PersistenceManager persistenceManager = (PersistenceManager) Class.forName(className).newInstance();
                configContext.setProperty(MercuryConstants.RM_PERSISTANCE_MANAGER, persistenceManager);
            } catch (InstantiationException e) {
                log.error("Can not instantiate persistence manager", e);
                throw new AxisFault("Can not instantiate persistence manager", e);
            } catch (IllegalAccessException e) {
                log.error("Illegal Access to persistence manager", e);
                throw new AxisFault("Illegal Access to persistence manager", e);
            } catch (ClassNotFoundException e) {
                log.error("Persistance manager class not found", e);
                throw new AxisFault("Persistance manager class not found", e);
            }
        }

        // start the sequence remover.
        SequenceRemovalWorker sequenceRemovalWorker = new SequenceRemovalWorker(configContext);
        sequenceRemovalWorker.setSequenceRemovalWorkerSleepTime(sequenceRemvalWorkerSleepTime);
        configContext.getThreadPool().execute(sequenceRemovalWorker);
    }

    public void engageNotify(AxisDescription axisDescription) throws AxisFault {
        //TODO: implement if necessary
    }

    public void shutdown(ConfigurationContext configurationContext) throws AxisFault {
        // remove context objects
        configurationContext.removeProperty(MercuryConstants.RMS_CONTEXT);
        configurationContext.removeProperty(MercuryConstants.RMD_CONTEXT);
    }

    public void applyPolicy(Policy policy, AxisDescription axisDescription) throws AxisFault {
        //TODO: implement if necessary
    }

    public boolean canSupportAssertion(Assertion assertion) {
        //TODO: implement if necessary
        return false;
    }
}
