/*
 * Copyright 2008,2009 WSO2, Inc. http://www.wso2.org.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wso2.mercury.workers;

import org.wso2.mercury.state.InvokerBuffer;
import org.wso2.mercury.handlers.MercuryOutHandler;
import org.apache.axis2.AxisFault;
import org.apache.axis2.Constants;
import org.apache.axis2.engine.AxisEngine;
import org.apache.axis2.util.MessageContextBuilder;
import org.apache.axis2.context.MessageContext;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * invoker worker works on the
 * Invoker buffer to invoke the messages in correct order.
 */

public class InvokerWorker implements Runnable{

    private static Log log = LogFactory.getLog(InvokerWorker.class);

    public static long INVOKER_WORKER_SLEEP_TIME = 4000;

    private InvokerBuffer invokerBuffer;

    public InvokerWorker(InvokerBuffer invokerBuffer) {
        this.invokerBuffer = invokerBuffer;
    }

    public void run() {

        while ((invokerBuffer.getState() != InvokerBuffer.STATE_COMPLETE) &&
                (System.currentTimeMillis() - this.invokerBuffer.getLastAccessTime()) < InvokerBuffer.TIMEOUT_TIME) {
            // do actions here.
            try {
                invokerBuffer.doActions();
            } catch (AxisFault axisFault) {
                log.error("Axis Fault has received to Invoker worker");
                if (axisFault.getFaultType() == Constants.APPLICATION_FAULT) {
                    try {
                        AxisEngine.sendFault(axisFault.getFaultMessageContext());
                    } catch (AxisFault e) {
                        log.error("Problem when sending the fault message", e);
                    }
                } else {
                    // TODO: what to do with these exceptions
                    log.error("Got a soap processing exception");
                }

            }
            try {
                Thread.sleep(INVOKER_WORKER_SLEEP_TIME);
            } catch (InterruptedException e) {}
        }
        // invoker is timed out
        log.info("Stopping the invoker worker ");
    }
}
